import numpy as np

# Example 1: With confounding
print('example 1: with confounding')
np.random.seed(1)
N = 50000
B = np.random.binomial(1, 0.5, size=(N,))

# Values of A depend on B
A = np.zeros(N, dtype=int)
A[B == 1] = np.random.binomial(1, 0.9, size=np.sum(B == 1))
A[B == 0] = np.random.binomial(1, 0.1, size=np.sum(B == 0))

# Values of T depend on A and B
T = np.zeros(N, dtype=int)
T[(A == 1) & (B == 1)] = np.random.binomial(1, 0.9, size=np.sum((A == 1) & (B == 1)))
T[(A == 1) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 1) & (B == 0)))
T[(A == 0) & (B == 1)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 1)))
T[(A == 0) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 0)))

# 1.1 Observing A=1
obs_idx = (A == 1)
print(f'average value of B when observing A==1: {np.mean(B[obs_idx])}')
print(f'average value of T when observing A==1: {np.mean(T[obs_idx])}')

# 1.2 Manipulating A=1 (do(A=1))
A = np.ones(N, dtype=int)  # intervene A
T = np.zeros(N, dtype=int)
T[(A == 1) & (B == 1)] = np.random.binomial(1, 0.9, size=np.sum((A == 1) & (B == 1)))
T[(A == 1) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 1) & (B == 0)))
T[(A == 0) & (B == 1)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 1)))
T[(A == 0) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 0)))

print(f'average value of B when manipulating A==1: {np.mean(B)}')
print(f'average value of T when manipulating A==1: {np.mean(T)}')
print('the difference in T when observing vs. manipulating A is a result of confounding by B.')
print('The Value of B when observing vs. manipulating A is different, when B is a cause of A')
print('This difference propagates to the difference in T, since B also causes T.')

# Example 2: Without confounding
print()
print('example 2: without confounding')
np.random.seed(1)
B = np.random.binomial(1, 0.5, size=(N,))
A = np.random.binomial(1, 0.5, size=(N,))  # A does not depend on B, but it has the same distribution as in example 1.

T = np.zeros(N, dtype=int)
T[(A == 1) & (B == 1)] = np.random.binomial(1, 0.9, size=np.sum((A == 1) & (B == 1)))
T[(A == 1) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 1) & (B == 0)))
T[(A == 0) & (B == 1)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 1)))
T[(A == 0) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 0)))

# Observing A=1
obs_idx = (A == 1)
print(f'average value of B when observing A==1: {np.mean(B[obs_idx])}')
print(f'average value of T when observing A==1: {np.mean(T[obs_idx])}')

# Manipulating A=1 (do(A=1))
A = np.ones(N, dtype=int)
T = np.zeros(N, dtype=int)
T[(A == 1) & (B == 1)] = np.random.binomial(1, 0.9, size=np.sum((A == 1) & (B == 1)))
T[(A == 1) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 1) & (B == 0)))
T[(A == 0) & (B == 1)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 1)))
T[(A == 0) & (B == 0)] = np.random.binomial(1, 0.1, size=np.sum((A == 0) & (B == 0)))

print(f'average value of B when manipulating A==1: {np.mean(B)}')
print(f'average value of T when manipulating A==1: {np.mean(T)}')

